@@ -18,6 +18,7 @@
 """
 The Bingham mechanism in differential privacy, for estimating the first eigenvector of a covariance matrix.
 """
+import secrets
 from numbers import Real
 
 import numpy as np
@@ -42,12 +43,17 @@ class Bingham(DPMechanism):
     sensitivity : float, default: 1
         The sensitivity of the mechanism.  Must be in [0, ∞).
 
+    random_state : int or RandomState, optional
+        Controls the randomness of the mechanism.  To obtain a deterministic behaviour during randomisation,
+        ``random_state`` has to be fixed to an integer.
+
     """
-    def __init__(self, *, epsilon, sensitivity=1.0):
-        super().__init__(epsilon=epsilon, delta=0)
+    def __init__(self, *, epsilon, sensitivity=1.0, random_state=None):
+        super().__init__(epsilon=epsilon, delta=0, random_state=random_state)
         self.sensitivity = self._check_sensitivity(sensitivity)
 
-        self._rng = np.random.default_rng()
+        if isinstance(self._rng, secrets.SystemRandom):
+            self._rng = np.random.default_rng()
 
     @classmethod
     def _check_epsilon_delta(cls, epsilon, delta):
