@@ -2,14 +2,10 @@ import numpy as np
 from unittest import TestCase
 
 from diffprivlib.mechanisms import GeometricFolded
-from diffprivlib.utils import global_seed
 
 
 class TestGeometricFolded(TestCase):
     def setup_method(self, method):
-        if method.__name__ .endswith("prob"):
-            global_seed(314159)
-
         self.mech = GeometricFolded
 
     def teardown_method(self, method):
@@ -85,7 +81,7 @@ class TestGeometricFolded(TestCase):
             mech.randomise(1.0)
 
     def test_zero_median_prob(self):
-        mech = self.mech(epsilon=1, sensitivity=1, lower=0, upper=4)
+        mech = self.mech(epsilon=1, sensitivity=1, lower=0, upper=4, random_state=0)
         vals = []
 
         for i in range(1000):
@@ -97,7 +93,7 @@ class TestGeometricFolded(TestCase):
     def test_neighbors_prob(self):
         epsilon = np.log(2)
         runs = 1000
-        mech = self.mech(epsilon=epsilon, sensitivity=1, lower=0, upper=4)
+        mech = self.mech(epsilon=epsilon, sensitivity=1, lower=0, upper=4, random_state=0)
         count = [0, 0]
 
         for i in range(runs):
@@ -112,6 +108,18 @@ class TestGeometricFolded(TestCase):
         self.assertGreater(count[0], count[1])
         self.assertLessEqual(count[0], np.exp(epsilon) * count[1] + 0.15 * runs)
 
+    def test_random_state(self):
+        mech1 = self.mech(epsilon=1, sensitivity=1, lower=0, upper=4, random_state=42)
+        mech2 = self.mech(epsilon=1, sensitivity=1, lower=0, upper=4, random_state=42)
+        self.assertEqual([mech1.randomise(0) for _ in range(100)], [mech2.randomise(0) for _ in range(100)])
+
+        self.assertNotEqual([mech1.randomise(0)] * 100, [mech1.randomise(0) for _ in range(100)])
+
+        rng = np.random.RandomState(0)
+        mech1 = self.mech(epsilon=1, sensitivity=1, lower=0, upper=4, random_state=rng)
+        mech2 = self.mech(epsilon=1, sensitivity=1, lower=0, upper=4, random_state=rng)
+        self.assertNotEqual([mech1.randomise(0) for _ in range(100)], [mech2.randomise(0) for _ in range(100)])
+
     def test_repr(self):
         repr_ = repr(self.mech(epsilon=1, sensitivity=1, lower=0, upper=4))
         self.assertIn(".GeometricFolded(", repr_)
