@@ -2,14 +2,10 @@ import numpy as np
 from unittest import TestCase
 
 from diffprivlib.mechanisms import LaplaceTruncated
-from diffprivlib.utils import global_seed
 
 
 class TestLaplaceTruncated(TestCase):
     def setup_method(self, method):
-        if method.__name__ .endswith("prob"):
-            global_seed(314159)
-
         self.mech = LaplaceTruncated
 
     def teardown_method(self, method):
@@ -52,7 +48,7 @@ class TestLaplaceTruncated(TestCase):
             mech.randomise("Hello")
 
     def test_zero_median_prob(self):
-        mech = self.mech(epsilon=1, delta=0, sensitivity=1, lower=0, upper=1)
+        mech = self.mech(epsilon=1, delta=0, sensitivity=1, lower=0, upper=1, random_state=0)
         vals = []
 
         for i in range(10000):
@@ -64,7 +60,7 @@ class TestLaplaceTruncated(TestCase):
     def test_neighbors_prob(self):
         epsilon = 1
         runs = 10000
-        mech = self.mech(epsilon=1, delta=0, sensitivity=1, lower=0, upper=1)
+        mech = self.mech(epsilon=1, delta=0, sensitivity=1, lower=0, upper=1, random_state=0)
         count = [0, 0]
 
         for i in range(runs):
@@ -91,6 +87,18 @@ class TestLaplaceTruncated(TestCase):
         self.assertTrue(np.all(vals >= 0))
         self.assertTrue(np.all(vals <= 1))
 
+    def test_random_state(self):
+        mech1 = self.mech(epsilon=1, sensitivity=1, lower=0, upper=4, random_state=42)
+        mech2 = self.mech(epsilon=1, sensitivity=1, lower=0, upper=4, random_state=42)
+        self.assertEqual([mech1.randomise(0) for _ in range(100)], [mech2.randomise(0) for _ in range(100)])
+
+        self.assertNotEqual([mech1.randomise(0)] * 100, [mech1.randomise(0) for _ in range(100)])
+
+        rng = np.random.RandomState(0)
+        mech1 = self.mech(epsilon=1, sensitivity=1, lower=0, upper=4, random_state=rng)
+        mech2 = self.mech(epsilon=1, sensitivity=1, lower=0, upper=4, random_state=rng)
+        self.assertNotEqual([mech1.randomise(0) for _ in range(100)], [mech2.randomise(0) for _ in range(100)])
+
     def test_repr(self):
         repr_ = repr(self.mech(epsilon=1, delta=0, sensitivity=1, lower=0, upper=1))
         self.assertIn(".LaplaceTruncated(", repr_)
