@@ -2,6 +2,7 @@ import numpy as np
 from unittest import TestCase
 
 from diffprivlib.mechanisms import PermuteAndFlip
+from diffprivlib.utils import check_random_state
 
 
 class TestPermuteAndFlip(TestCase):
@@ -155,15 +156,16 @@ class TestPermuteAndFlip(TestCase):
         self.assertRaises(ValueError, self.mech, epsilon=1, utility=list_with_inf, sensitivity=1)
         # self.assertRaises(ValueError, self.mech, epsilon=1, utility=[1] * 3, measure=list_with_inf, sensitivity=1)
 
-        self.assertRaises(ValueError, self.mech, epsilon=1, utility=[-l for l in list_with_inf], sensitivity=1)
-        # self.assertRaises(ValueError, self.mech, epsilon=1, utility=[1] * 3, measure=[-l for l in list_with_inf],
+        self.assertRaises(ValueError, self.mech, epsilon=1, utility=[-val for val in list_with_inf], sensitivity=1)
+        # self.assertRaises(ValueError, self.mech, epsilon=1, utility=[1] * 3, measure=[-val for val in list_with_inf],
         #                   sensitivity=1)
 
     def test_distrib_prob(self):
         epsilon = np.log(2)
         runs = 20000
-        mech1 = self.mech(epsilon=epsilon, utility=[2, 1, 0], sensitivity=1, monotonic=False)
-        mech2 = self.mech(epsilon=epsilon, utility=[2, 1, 1], sensitivity=1, monotonic=False)
+        rng = np.random.RandomState(42)
+        mech1 = self.mech(epsilon=epsilon, utility=[2, 1, 0], sensitivity=1, monotonic=False, random_state=rng)
+        mech2 = self.mech(epsilon=epsilon, utility=[2, 1, 1], sensitivity=1, monotonic=False, random_state=rng)
         counts = np.zeros((2, 3))
 
         for i in range(runs):
@@ -176,9 +178,10 @@ class TestPermuteAndFlip(TestCase):
 
     def test_monotonic_distrib(self):
         epsilon = np.log(2)
-        runs = 40000
-        mech1 = self.mech(epsilon=epsilon, utility=[2, 1, 0], sensitivity=1, monotonic=True)
-        mech2 = self.mech(epsilon=epsilon, utility=[2, 1, 1], sensitivity=1, monotonic=True)
+        runs = 1000
+        rng = check_random_state(0)
+        mech1 = self.mech(epsilon=epsilon, utility=[2, 1, 0], sensitivity=1, monotonic=True, random_state=rng)
+        mech2 = self.mech(epsilon=epsilon, utility=[2, 1, 1], sensitivity=1, monotonic=True, random_state=rng)
         counts = np.zeros((2, 3))
 
         for i in range(runs):
@@ -186,9 +189,18 @@ class TestPermuteAndFlip(TestCase):
             counts[1, mech2.randomise()] += 1
 
         for vec in counts.T:
-            # print(vec.max() / vec.min())
             self.assertLessEqual(vec.max() / vec.min(), np.exp(epsilon) + 0.1)
 
+    def test_random_state(self):
+        mech1 = self.mech(epsilon=1, utility=[2, 1, 0], sensitivity=1, random_state=42)
+        mech2 = self.mech(epsilon=1, utility=[2, 1, 0], sensitivity=1, random_state=42)
+        self.assertEqual([mech1.randomise() for _ in range(100)], [mech2.randomise() for _ in range(100)])
+
+        self.assertNotEqual([mech1.randomise()] * 100, [mech1.randomise() for _ in range(100)])
+
+        mech2 = self.mech(epsilon=1, utility=[2, 1, 0], sensitivity=1, random_state=np.random.RandomState(0))
+        self.assertNotEqual([mech1.randomise() for _ in range(100)], [mech2.randomise() for _ in range(100)])
+
     def test_repr(self):
         repr_ = repr(self.mech(epsilon=1, utility=[1], sensitivity=1))
         self.assertIn(".PermuteAndFlip(", repr_)
