@@ -3,9 +3,6 @@ from unittest import TestCase
 
 from diffprivlib.mechanisms import Geometric
 from diffprivlib.mechanisms.transforms import StringToInt
-from diffprivlib.utils import global_seed
-
-global_seed(3141592653)
 
 
 class TestStringToInt(TestCase):
