@@ -4,6 +4,7 @@ import numpy as np
 import pytest
 
 from diffprivlib.tools.quantiles import percentile
+from diffprivlib.utils import check_random_state
 
 
 class TestPercentile(TestCase):
@@ -19,9 +20,10 @@ class TestPercentile(TestCase):
         self.assertRaises(ValueError, percentile, a, [50] * 3 + [-1])
 
     def test_simple(self):
-        a = np.random.random(1000)
+        rng = check_random_state(0)
+        a = rng.random(1000)
 
-        res = percentile(a, 50, epsilon=5, bounds=(0, 1))
+        res = percentile(a, 50, epsilon=5, bounds=(0, 1), random_state=rng)
         self.assertAlmostEqual(res, 0.5, delta=0.05)
 
     @pytest.mark.filterwarnings("ignore:Bounds have not been specified")
